@@ -25,6 +25,8 @@
 //! \details    render copy implement file
 //!
 
+#include "vpkrnheader.h"
+#include "vp_common_defs.h"
 #include "media_render_copy_next.h"
 #include "media_interfaces_mhw_next.h"
 #include "media_render_common.h"
@@ -40,6 +42,7 @@
         (MhwKernelParam).iSize    = (_pKernelEntry)->iSize;                         \
         (MhwKernelParam).iKUID    = (_pKernelEntry)->iKUID;                         \
         (MhwKernelParam).iKCID    = (_pKernelEntry)->iKCID;                         \
+        (MhwKernelParam).iPaddingSize = (_pKernelEntry)->iPaddingSize;              \
     } while(0)
 
 
@@ -117,7 +120,9 @@ MOS_STATUS RenderCopyStateNext::Initialize()
     RenderHalSettings.iMediaStates = 32;
     MCPY_CHK_STATUS_RETURN(m_renderHal->pfnInitialize(m_renderHal, &RenderHalSettings));
 
-    m_renderHal->sseuTable = defaultSSEUTable;
+    m_renderHal->sseuTable              = defaultSSEUTable;
+    m_renderHal->forceDisablePreemption = true;
+
     return MOS_STATUS_SUCCESS;
 }
 
@@ -213,7 +218,6 @@ MOS_STATUS RenderCopyStateNext::SetupKernel(
 
     pcKernelBin = m_KernelBin;
     dwKernelBinSize = m_KernelBinSize;
-    MCPY_NORMALMESSAGE("kernel name %s , kernel size %d", *(char *)pcKernelBin, dwKernelBinSize);
 
     if (nullptr == m_pKernelBin)
     {
@@ -302,28 +306,6 @@ MOS_STATUS RenderCopyStateNext::SubmitCMD()
     // Register the resource of GSH
     MCPY_CHK_STATUS_RETURN(pRenderHal->pfnReset(pRenderHal));
 
-    // Register the input resource;
-    MCPY_CHK_STATUS_RETURN(pOsInterface->pfnRegisterResource(
-        pOsInterface,
-        (PMOS_RESOURCE)&m_Source.OsResource,
-        true,
-        true));
-
-    // Ensure input can be read
-    pOsInterface->pfnSyncOnResource(
-        pOsInterface,
-        &m_Source.OsResource,
-        pOsInterface->CurrentGpuContextOrdinal,
-        false);
-
-    // Ensure Output can be read
-    pOsInterface->pfnSyncOnResource(
-        pOsInterface,
-        &m_Target.OsResource,
-        pOsInterface->CurrentGpuContextOrdinal,
-        false);
-
-
     // Set copy kernel
     SetupKernel(m_currKernelId);
 
@@ -393,9 +375,7 @@ MOS_STATUS RenderCopyStateNext::SubmitCMD()
 
     // Set Perf Tag
     pOsInterface->pfnResetPerfBufferID(pOsInterface);
-    pOsInterface->pfnSetPerfTag(
-        pOsInterface,
-        pRenderData->PerfTag);
+    m_osInterface->pfnSetPerfTag(m_osInterface, RENDER_COPY);
 
     // Setup Compute Walker
     pWalkerParams = nullptr;
@@ -519,7 +499,8 @@ MOS_STATUS RenderCopyStateNext::SetupSurfaceStates()
     int32_t                         iBTEntry;
     PRENDERHAL_INTERFACE            pRenderHal  = m_renderHal;
     PMEDIACOPY_RENDER_DATA          pRenderData = &m_RenderData;
-
+    RENDERHAL_SURFACE               RenderHalSource = {};  // source for mhw
+    RENDERHAL_SURFACE               RenderHalTarget = {};
     MCPY_CHK_NULL_RETURN(pRenderHal);
     MCPY_CHK_NULL_RETURN(pRenderData);
     // Source surface
@@ -527,7 +508,7 @@ MOS_STATUS RenderCopyStateNext::SetupSurfaceStates()
 
     pRenderData->SurfMemObjCtl.SourceSurfMemObjCtl =
          pRenderHal->pOsInterface->pfnCachePolicyGetMemoryObject(
-         MOS_HW_RESOURCE_USAGE_VP_INTERNAL_READ_WRITE_RENDER,
+         MOS_MP_RESOURCE_USAGE_SurfaceState_RCS,
          pRenderHal->pOsInterface->pfnGetGmmClientContext(pRenderHal->pOsInterface)).DwordValue;
 
     pRenderData->SurfMemObjCtl.TargetSurfMemObjCtl = pRenderData->SurfMemObjCtl.SourceSurfMemObjCtl;
@@ -544,7 +525,7 @@ MOS_STATUS RenderCopyStateNext::SetupSurfaceStates()
 
     if (Format_NV12 == m_Target.Format)
     {
-         // set NV12 as 2 plane
+        // set NV12 as 2 plane because render copy only uses DP reading&writing.
         RenderHalSource.SurfType = RENDERHAL_SURF_OUT_RENDERTARGET;
         RenderHalTarget.SurfType = RENDERHAL_SURF_OUT_RENDERTARGET;
     }
